import httpx
import asyncio
from mcp.server.fastmcp import FastMCP
import logging
import os
import base64
from typing import Optional, Dict, Union, Any, List
from enum import IntEnum, Enum
import re
from pydantic import BaseModel, Field

# Set up logging
logging.basicConfig(level=logging.INFO)

# Initialize FastMCP server
mcp = FastMCP("freshrelease-mcp")

FRESHRELEASE_API_KEY = os.getenv("FRESHRELEASE_API_KEY")
FRESHRELEASE_DOMAIN = os.getenv("FRESHRELEASE_DOMAIN")


def parse_link_header(link_header: str) -> Dict[str, Optional[int]]:
    """Parse the Link header to extract pagination information.

    Args:
        link_header: The Link header string from the response

    Returns:
        Dictionary containing next and prev page numbers
    """
    pagination = {
        "next": None,
        "prev": None
    }

    if not link_header:
        return pagination

    # Split multiple links if present
    links = link_header.split(',')

    for link in links:
        # Extract URL and rel
        match = re.search(r'<(.+?)>;\s*rel="(.+?)"', link)
        if match:
            url, rel = match.groups()
            # Extract page number from URL
            page_match = re.search(r'page=(\d+)', url)
            if page_match:
                page_num = int(page_match.group(1))
                pagination[rel] = page_num

    return pagination

# Status categories from Freshrelease repository
class STATUS_CATEGORIES(str, Enum):
    todo = 1
    in_progress = 2
    done = 3

class STATUS_CATEGORY_NAMES(str, Enum):
    YET_TO_START = "Yet To Start"
    WORK_IN_PROGRESS = "Work In Progress"
    COMPLETED = "Completed"

@mcp.tool()
async def fr_create_project(name: str, description: Optional[str] = None) -> Dict[str, Any]:
    """Create a project in Freshrelease."""
    if not FRESHRELEASE_DOMAIN or not FRESHRELEASE_API_KEY:
        return {"error": "FRESHRELEASE_DOMAIN or FRESHRELEASE_API_KEY is not set"}

    base_url = f"https://{FRESHRELEASE_DOMAIN}"
    url = f"{base_url}/projects"
    headers = {
        "Authorization": f"Token {FRESHRELEASE_API_KEY}",
        "Content-Type": "application/json",
    }
    payload: Dict[str, Any] = {"name": name}
    if description is not None:
        payload["description"] = description

    async with httpx.AsyncClient() as client:
        try:
            response = await client.post(url, headers=headers, json=payload)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            return {"error": f"Failed to create project: {str(e)}", "details": e.response.json() if e.response else None}
        except Exception as e:
            return {"error": f"An unexpected error occurred: {str(e)}"}


@mcp.tool()
async def fr_get_project(project_identifier: Union[int, str]) -> Dict[str, Any]:
    """Get a project from Freshrelease by ID or key.

    - project_identifier: numeric ID (e.g., 123) or key (e.g., "ENG")
    """
    if not FRESHRELEASE_DOMAIN or not FRESHRELEASE_API_KEY:
        return {"error": "FRESHRELEASE_DOMAIN or FRESHRELEASE_API_KEY is not set"}

    base_url = f"https://{FRESHRELEASE_DOMAIN}"
    url = f"{base_url}/projects/{project_identifier}"
    headers = {
        "Authorization": f"Token {FRESHRELEASE_API_KEY}",
        "Content-Type": "application/json",
    }

    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(url, headers=headers)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            return {"error": f"Failed to fetch project: {str(e)}", "details": e.response.json() if e.response else None}
        except Exception as e:
            return {"error": f"An unexpected error occurred: {str(e)}"}


@mcp.tool()
async def fr_create_task(
    project_identifier: Union[int, str],
    title: str,
    description: Optional[str] = None,
    assignee_id: Optional[int] = None,
    status: Optional[str] = None,
    due_date: Optional[str] = None,
) -> Dict[str, Any]:
    """Create a task under a Freshrelease project.

    - due_date: ISO 8601 date string (e.g., 2025-12-31) if supported by your account
    """
    if not FRESHRELEASE_DOMAIN or not FRESHRELEASE_API_KEY:
        return {"error": "FRESHRELEASE_DOMAIN or FRESHRELEASE_API_KEY is not set"}

    base_url = f"https://{FRESHRELEASE_DOMAIN}"
    url = f"{base_url}/{project_identifier}/issues"
    headers = {
        "Authorization": f"Token {FRESHRELEASE_API_KEY}",
        "Content-Type": "application/json",
    }

    payload: Dict[str, Any] = {"title": title}
    if description is not None:
        payload["description"] = description
    if assignee_id is not None:
        payload["assignee_id"] = assignee_id
    if status is not None:
        payload["status"] = status
    if due_date is not None:
        payload["due_date"] = due_date

    async with httpx.AsyncClient() as client:
        try:
            response = await client.post(url, headers=headers, json=payload)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            return {"error": f"Failed to create task: {str(e)}", "details": e.response.json() if e.response else None}
        except Exception as e:
            return {"error": f"An unexpected error occurred: {str(e)}"}


@mcp.tool()
async def fr_get_task(project_identifier: Union[int, str],task_id: int) -> Dict[str, Any]:
    """Get a task from Freshrelease by ID."""
    if not FRESHRELEASE_DOMAIN or not FRESHRELEASE_API_KEY:
        return {"error": "FRESHRELEASE_DOMAIN or FRESHRELEASE_API_KEY is not set"}

    base_url = f"https://{FRESHRELEASE_DOMAIN}"
    url = f"{base_url}/{project_identifier}/issues/{task_id}"
    headers = {
        "Authorization": f"Token {FRESHRELEASE_API_KEY}",
        "Content-Type": "application/json",
    }

    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(url, headers=headers)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            return {"error": f"Failed to fetch task: {str(e)}", "details": e.response.json() if e.response else None}
        except Exception as e:
            return {"error": f"An unexpected error occurred: {str(e)}"}

@mcp.tool()
async def fr_get_all_tasks(project_identifier: Union[int, str]) -> Dict[str, Any]:
    """Get a task from Freshrelease by ID."""
    if not FRESHRELEASE_DOMAIN or not FRESHRELEASE_API_KEY:
        return {"error": "FRESHRELEASE_DOMAIN or FRESHRELEASE_API_KEY is not set"}

    base_url = f"https://{FRESHRELEASE_DOMAIN}"
    url = f"{base_url}/{project_identifier}/issues"
    headers = {
        "Authorization": f"Token {FRESHRELEASE_API_KEY}",
        "Content-Type": "application/json",
    }

    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(url, headers=headers)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            return {"error": f"Failed to fetch task: {str(e)}", "details": e.response.json() if e.response else None}
        except Exception as e:
            return {"error": f"An unexpected error occurred: {str(e)}"}

def main():
    logging.info("Starting Freshdesk MCP server")
    mcp.run(transport='stdio')

if __name__ == "__main__":
    main()



import unittest
from unittest.mock import patch, MagicMock
import json
import os
import sys
import asyncio

# Add the parent directory to the path so we can import the module
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from src.freshrelease_mcp.server import parse_link_header

# Create a mock version of our server functions to isolate testing
async def mock_list_companies(page=1, per_page=30):
    companies = [
        {
            "id": 51001000001,
            "name": "Acme Corp",
            "description": "Leading provider of widgets",
            "note": "Preferred vendor",
            "domains": ["acme.example"],
            "created_at": "2024-05-01T12:00:00Z",
            "updated_at": "2025-01-15T09:30:00Z",
            "custom_fields": {
                "organisation_name": "Acme Corp",
                "account_status": "Active",
                "hosting_platform": "AWS"
            }
        },
        {
            "id": 51001000002,
            "name": "Globex Inc",
            "domains": ["globex.com", "globex.org"],
            "created_at": "2024-11-10T08:45:00Z",
            "updated_at": "2025-02-20T16:10:00Z",
            "custom_fields": {
                "organisation_name": "Globex Inc",
                "account_status": "Expired"
            }
        }
    ]

    pagination_info = {
        "next": 2 if page < 3 else None,
        "prev": page - 1 if page > 1 else None
    }

    return {
        "companies": companies,
        "pagination": {
            "current_page": page,
            "next_page": pagination_info.get("next"),
            "prev_page": pagination_info.get("prev"),
            "per_page": per_page
        }
    }

async def mock_view_company(company_id):
    if company_id == 51001000001:
        return {
            "id": 51001000001,
            "name": "Acme Corp",
            "description": "Leading provider of widgets",
            "note": "Preferred vendor",
            "domains": ["acme.example"],
            "created_at": "2024-05-01T12:00:00Z",
            "updated_at": "2025-01-15T09:30:00Z",
            "custom_fields": {
                "organisation_name": "Acme Corp",
                "account_status": "Active",
                "hosting_platform": "AWS"
            }
        }
    else:
        return {"error": "Company not found"}

async def mock_search_companies(query):
    if "acme" in query.lower():
        return [
            {
                "id": 51001000001,
                "name": "Acme Corp"
            },
            {
                "id": 51001000002,
                "name": "Acme Solutions"
            }
        ]
    else:
        return []

async def mock_list_company_fields():
    return [
        {
            "id": 51000152653,
            "name": "name",
            "label": "Company Name",
            "position": 1,
            "required_for_agents": True,
            "type": "default_name",
            "default": True
        },
        {
            "id": 51000169767,
            "name": "organisation_name",
            "label": "Organisation Name",
            "position": 2,
            "required_for_agents": True,
            "type": "custom_text",
            "default": False
        },
        {
            "id": 51000265522,
            "name": "account_status",
            "label": "Account Status",
            "position": 3,
            "required_for_agents": False,
            "type": "custom_dropdown",
            "default": False,
            "choices": [
                "Active",
                "Expired"
            ]
        }
    ]

# Class for sync tests using unittest
class TestParseHeaderFunction(unittest.TestCase):
    def test_parse_link_header(self):
        # Test the parse_link_header function directly
        header = '<https://example.com/page=2>; rel="next", <https://example.com/page=1>; rel="prev"'
        result = parse_link_header(header)
        self.assertEqual(result.get('next'), 2)
        self.assertEqual(result.get('prev'), 1)

    def test_parse_link_header_empty(self):
        # Test with empty header
        result = parse_link_header("")
        self.assertEqual(result, {"next": None, "prev": None})

    def test_parse_link_header_invalid_format(self):
        # Test with invalid format
        result = parse_link_header("invalid format")
        self.assertEqual(result, {"next": None, "prev": None})

# Define async test cases outside of unittest framework
async def test_list_companies():
    result = await mock_list_companies(page=1, per_page=10)

    assert 'companies' in result
    assert len(result['companies']) == 2
    assert result['companies'][0]['name'] == 'Acme Corp'
    assert 'pagination' in result
    assert result['pagination']['current_page'] == 1
    assert 'next_page' in result['pagination']
    print("✓ test_list_companies passed")

async def test_view_company():
    result = await mock_view_company(51001000001)

    assert result['id'] == 51001000001
    assert result['name'] == 'Acme Corp'
    assert result['domains'] == ['acme.example']
    print("✓ test_view_company passed")

async def test_search_companies():
    result = await mock_search_companies("acme")

    assert len(result) == 2
    assert result[0]['id'] == 51001000001
    assert result[0]['name'] == 'Acme Corp'
    print("✓ test_search_companies passed")

async def test_list_company_fields():
    result = await mock_list_company_fields()

    assert len(result) == 3
    assert result[0]['name'] == 'name'
    assert result[1]['name'] == 'organisation_name'
    assert result[2]['name'] == 'account_status'
    print("✓ test_list_company_fields passed")

if __name__ == "__main__":
    # Run async tests
    print("Running async tests:")
    asyncio.run(test_list_companies())
    asyncio.run(test_view_company())
    asyncio.run(test_search_companies())
    asyncio.run(test_list_company_fields())

    # Run sync tests
    print("\nRunning sync tests:")
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


import asyncio
from freshrelease_mcp.server import (
    fr_list_status_categories,
    fr_get_status_category_id,
    fr_get_status_category_id_from_name,
    fr_list_status_category_names,
    fr_create_project,
    fr_get_project,
    fr_create_task,
    fr_get_task,
)

async def test_fr_list_status_categories():
    result = await fr_list_status_categories()
    print(result)

async def test_fr_get_status_category_id():
    result = await fr_get_status_category_id("todo")
    print(result)

async def test_fr_get_status_category_id_from_name():
    result = await fr_get_status_category_id_from_name("Yet To Start")
    print(result)

async def test_fr_list_status_category_names():
    result = await fr_list_status_category_names()
    print(result)

async def test_fr_create_project():
    # Adjust parameters or mock environment as needed
    result = await fr_create_project("Demo Project", description="Example")
    print(result)

async def test_fr_get_project():
    # Replace with a valid key or ID in your environment
    result = await fr_get_project("ENG")
    print(result)

async def test_fr_create_task():
    # Replace with a valid key or ID in your environment
    result = await fr_create_task("ENG", "Demo Task", description="Task details")
    print(result)

async def test_fr_get_task():
    # Replace with a valid task ID in your environment
    result = await fr_get_task(123)
    print(result)

if __name__ == "__main__":
    # asyncio.run(test_fr_list_status_categories())
    # asyncio.run(test_fr_get_status_category_id())
    # asyncio.run(test_fr_get_status_category_id_from_name())
    # asyncio.run(test_fr_list_status_category_names())
    # asyncio.run(test_fr_create_project())
    # asyncio.run(test_fr_get_project())
    # asyncio.run(test_fr_create_task())
    # asyncio.run(test_fr_get_task())
    pass